%MATLAB Script-File zubept2_168.m
%Zustandsregelung mit Zustandsbeobachter fr PT2-Strecke
clf								  %Graphisches Fenster lschen
KS=1; w0S=1; DS=0.707;            %Streckenparameter
numGS=KS*w0S^2;                   %Zhlerpolynom der Strecke
denGS=[1 2*DS*w0S w0S^2];         %Nennerpolynom der Strecke
sys=ss(tf(numGS,denGS));		  %GS(s) --> Zustandsmodell
[sysR,TR]=T_RNFORM_168(sys); 	  %Zustandsmodell in 
                                  %Regelungsnormalform
w0Z=2*w0S;                        %w0 der Zustandsregelung
sp1Z=w0Z*(-DS+j*sqrt(1-DS^2));    %Pole der
sp2Z=w0Z*(-DS-j*sqrt(1-DS^2));    %Zustandsregelung
poleZ=[sp1Z,sp2Z];                %Zeilenvektor der Vorgabepole
AR=sysR.a;						  %Systemmatrix von sysR
bR=sysR.b;						  %Eingangsvektor von sysR
rR=place(AR,bR,poleZ);            %Rckfhrkoeffizienten
rR=-rR;                           %sind positiv
vwR=(w0S^2-rR(1))/(KS*w0S^2);     %Vorfilter

[sysB,TB]=T_BNFORM_168(sys); 	  %Zustandsmodell in
       %Beobachtungsnormalform mit Function-File T_BNFORM_1.m
w0B=2*w0Z;                        %w0 des Zustandsbeobachters
sp1B=w0B*(-DS+j*sqrt(1-DS^2));    %Pole des
sp2B=w0B*(-DS-j*sqrt(1-DS^2));    %Zustandsbeobachters
poleB=[sp1B,sp2B];                %Zeilenvektor der Vorgabepole
AB=sysB.a;						  %Systemmatrix von sysB
cB=sysB.c;						  %Ausgangsvektor von sysB
lB=place(AB',cB',poleB);          %Beobachtungsvektor
lB=lB';                           %lB wird transponiert
%TB_1 ist Inverse der Matrix zur Transformation auf
                                  %Beobachtungsnormalform
%TR ist die Matrix zur Transformation auf Regelungsnormalform
TB_1=inv(TB); TRB=TR*TB_1;
%Zustandsmodell der Zustandsregelung mit Zustandsbeobachter:
cR=sysR.c;						  %Ausgangsvektor von sysR
bB=sysB.b;						  %Eingangsvektor von sysB
AS=[AR, bR*rR*TRB; lB*cR, AB-lB*cB+bB*rR*TRB]; %Systemmatrix
BS=[bR*vwR; bB*vwR];              %Eingangsmatrix
CS=[cR, zeros(size(cB))];         %Ausgangsmatrix
DS=0;                             %Durchgangsmatrix

x0R=[0.25; -0.5];                 %Anfangswerte der Regelung
x0B=[0;0];                        %Anfangswerte des Beobachters
x0S=[x0R;x0B];                    %Matrix der Anfangswerte

t=0:0.01:6;                       %Vektor der Zeitwerte
XS=zeros(length(t),4);            %Vordefinition Matrix XS
[YS,XS]=step(AS,BS,CS,DS,1,t);    %Sprungantwort
for k=1:length(t)
  xSS=expm(AS*t(k))*x0S;          %Matrix-e-Funktion
  XS(k,:)=xSS'+XS(k,:);
  XS(k,[3 4])=XS(k,[3 4])*TRB';
end

subplot(211)
plot(t,XS(:,1),t,XS(:,3))         %XS(:,1) ist x1R
grid                              %XS(:,3) ist x^1R
xlabel('\it{Zeit t/s}')           %Beschriftung der Abszisse
ylabel('{\itx}_{1R}{\it(t), x}_{1B}{\it(t)}') %" der Ordinate
title('\it{Einheits-Sprungantwort}') %Titel der Graphik
text(0.2,0.1,'{\itx}_{1B}')       %Bezeichnung der Kurve
text(0.6,0.3,'{\itx}_{1R}')

subplot(212)
plot(t,XS(:,2),t,XS(:,4))         %XS(:,2) ist x2R
grid                              %XS(:,4) ist x^2R
xlabel('\it{Zeit t/s}')           %Beschriftung der Abszisse
ylabel('{\itx}_{2R}{\it(t), x}_{2B}{\it(t)}') %" der Ordinate
title('\it{Einheits-Sprungantwort}') %Titel der Graphik
text(0.4,0.9,'{\itx}_{2B}')       %Bezeichnung der Kurve
text(0.3,0.1,'{\itx}_{2R}')
